require 'backport_refinements'
using OpsManagerUiDrivers::BackportRefinements

require 'json'
require 'opsmgr/api/http_client'
require 'opsmgr/api/results'
require 'opsmgr/settings/microbosh/product_list'

module Opsmgr
  module Api
    class Client
      attr_reader :environment_name

      def initialize(environment, om_version)
        @http_client = HttpClient.build(environment, om_version)
        @environment_name = environment.settings.dig('name').freeze
      end

      def installation_settings
        response = http_client.installation_settings
        if response.code == '200'
          InstallationSettingsResult.new(JSON.parse(response.body))
        else
          Error.new('Error viewing current installation settings', response)
        end
      end

      # .pivotal file is referred to as a "product" but the Tempest developers call it a "component"
      def upload_component(path)
        response = http_client.upload_component(path)
        basic_success_or_error("Error uploading #{path}", response)
      end

      def add_product(product_name, version)
        response = http_client.add_product(product_name, version)
        basic_success_or_error("Error adding '#{product_name} #{version}'", response)
      end

      def root_ca_certificate
        response = http_client.root_ca_certificate
        if response.code == '200'
          JSON.parse(response.body)['root_ca_certificate_pem']
        else
          Error.new('Error retrieving the root ca certificate', response)
        end
      end

      def upgrade_product(product_guid, to_version)
        response = http_client.upgrade_product(product_guid, to_version)
        basic_success_or_error("Error upgrading '#{product_guid}' to '#{to_version}'", response)
      end

      def download_staged_manifest(product_guid)
        response = http_client.download_staged_manifest(product_guid)
        if response.code == '200'
          StagedManifestResult.new(response.body)
        else
          Error.new("Error downloading staged manifest for '#{product_guid}'", response)
        end
      end

      def list_products
        response = http_client.list_products
        if response.code == '200'
          ListProductsResult.new(JSON.parse(response.body))
        else
          Error.new('Error listing products', response)
        end
      end

      def installed_products
        response = http_client.installed_products
        if response.code == '200'
          InstalledProductsResult.new(Opsmgr::Settings::Microbosh::ProductList.new(JSON.parse(response.body)))
        else
          Error.new('Error listing installed products', response)
        end
      end

      def delete_unused_products
        response = http_client.delete_unused_products
        basic_success_or_error('There was an error deleting the unused products:', response)
      end

      def basic_success_or_error(message, response)
        if response.code == '200'
          Result.new
        else
          Error.new(message, response)
        end
      end

      def import_installation(path, password)
        response = http_client.import_installation(path, password)
        basic_success_or_error("Error importing #{path}", response)
      end

      def import_stemcell(path)
        response = http_client.import_stemcell(path)
        basic_success_or_error("Error importing stemcell #{path}", response)
      end

      private

      attr_reader :http_client
    end
  end
end
# Copyright (c) 2014-2015 Pivotal Software, Inc.
# All rights reserved.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
# INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
# USE OR OTHER DEALINGS IN THE SOFTWARE.
