# rubocop:disable Metrics/ClassLength
require 'backport_refinements'
using OpsManagerUiDrivers::BackportRefinements

require 'net/http'
require 'net/http/post/multipart'
require 'json'
require 'open4'
require 'opsmgr/log'
require 'opsmgr/api/version20/endpoints'
require 'uaa'

# noinspection RubyResolve
module Opsmgr
  module Api
    class HttpClient
      include Loggable
      attr_reader :uri, :environment, :endpoints

      def self.build(environment, om_version)
        new(environment, Opsmgr::Api::Version20::Endpoints.new, om_version)
      end

      def initialize(environment, endpoints, om_version)
        @environment = environment
        @uri = URI.parse(environment.settings.dig('ops_manager', 'url'))
        @endpoints = endpoints
        @om_version = om_version
      end

      def upload_product_installation_settings(settings_file_path)
        req = Net::HTTP::Post::Multipart.new(
          endpoints.installation_settings_post_path,
          'installation[file]' =>
            UploadIO.new(
              settings_file_path,
              'application/octet-stream',
              File.basename(settings_file_path)
            )
        )

        add_auth_header(req)

        http.request(req)
      end

      def upload_component(product_path)
        # We ran into a possible Net::HTTP bug where Ruby would always drop the connection while waiting for the
        # uploaded zip file to unzip. Curl behaved correctly, but curb (Ruby bindings for libcurl) did not behave
        # well with WebMock, so here we are.
        upload_component_command =
          %W(
            curl -k
            --silent
            --fail
            #{uri}#{endpoints.upload_product_path}
            -F #{endpoints.upload_product_form_key}[file]=@#{product_path}
            -X POST
          )

        if Gem::Version.new(@om_version) >= Gem::Version.new('1.7')
          log.info('using oauth')
          token = get_oauth_token(web_auth_user)
          upload_component_command.push("-H")
          upload_component_command.push("Authorization: #{token}")
        else
          log.info('using basic auth')
          upload_component_command.push("-u")
          upload_component_command.push("#{web_auth_user}:#{web_auth_password}")
        end

        log.info('uploading product')

        error = nil
        status =
          Open4.popen4(*upload_component_command) do |_, _, stdout, stderr|
            log.info(stdout.read)
            error = stderr.read
          end

        if status.success?
          FakeResponse.new('200', '{}')
        else
          error_code = (error.match(/^\< HTTP\/.* (\d+) /) || [nil, '???'])[1]
          FakeResponse.new(error_code, error)
        end
      end

      def root_ca_certificate
        req = Net::HTTP::Get.new(endpoints.root_ca_certificate_path)
        add_auth_header(req)

        http.request(req)
      end

      def add_product(product_name, version)
        req = Net::HTTP::Post.new(endpoints.add_product_path)
        req.set_form_data('name' => product_name, 'product_version' => version)
        add_auth_header(req)

        http.request(req)
      end

      def upgrade_product(product_guid, to_version)
        req = Net::HTTP::Put.new(endpoints.upgrade_product_path(product_guid))
        req.set_form_data('to_version' => to_version)
        add_auth_header(req)

        http.request(req)
      end

      def download_staged_manifest(product_guid)
        req = Net::HTTP::Get.new(endpoints.download_staged_manifest(product_guid))
        add_auth_header(req)

        http.request(req)
      end

      def trigger_install
        req = Net::HTTP::Post.new(endpoints.install_post_path)
        req.body = ''
        add_auth_header(req)

        http.request(req)
      end

      # components are both uploaded AND configured AND deployed
      def list_components
        get_with_basic_auth(endpoints.list_components_path)
      end

      # products are uploaded but not necessarily configured or deployed
      def list_products
        get_with_basic_auth(endpoints.list_products_path)
      end

      # products that have been uploaded, configured, and installed
      def installed_products
        get_with_basic_auth(endpoints.installed_products_path)
      end

      def product_manifest(product_guid)
        get_with_basic_auth(endpoints.product_manifest_path(product_guid))
      end

      def installation_settings
        get_with_basic_auth(endpoints.installation_settings_get_path)
      end

      def installation_delete
        req = Net::HTTP::Delete.new(endpoints.installation_path)
        add_auth_header(req)

        http.request(req)
      end

      def installation_deletion_status
        get_with_basic_auth(endpoints.installation_deletion_status_path)
      end

      def create_the_first_user
        if Gem::Version.new(@om_version) >= Gem::Version.new('1.6')
          req = Net::HTTP::Post.new(endpoints.api_setup_path)
          req.set_form_data(
            'setup[user_name]' => web_auth_user,
            'setup[password]' => web_auth_password,
            'setup[password_confirmation]' => web_auth_password,
            'setup[eula_accepted]' => true
          )
        else
          req = Net::HTTP::Post.new(endpoints.api_user_path)
          req.set_form_data(
            'user[user_name]' => web_auth_user,
            'user[password]' => web_auth_password,
            'user[password_confirmation]' => web_auth_password,
          )
        end

        http.request(req)
      end

      def install_status(install_id)
        get_with_basic_auth(endpoints.show_installation_status(install_id))
      end

      def install_log(install_id)
        get_with_basic_auth(endpoints.installation_log_path(install_id))
      end

      def mark_product_for_deletion(product_guid)
        req = Net::HTTP::Delete.new(endpoints.mark_for_deletion_path(product_guid))
        add_auth_header(req)

        http.request(req)
      end

      def delete_unused_products
        req = Net::HTTP::Delete.new(endpoints.list_products_path)
        add_auth_header(req)

        http.request(req)
      end

      def version
        endpoints.version
      end

      def import_installation(installation_file, password)
        create_the_first_user unless Gem::Version.new(@om_version) >= Gem::Version.new('1.7')

        log.info('uploading the installation file')
        req = Net::HTTP::Post::Multipart.new(
          endpoints.import_installation_path,
          'installation[file]' =>
            UploadIO.new(
              installation_file,
              'application/octet-stream',
              File.basename(installation_file)
            ),
          import_secret_parameter_name => password
        )

        # The post-1.7 import happens before a user has been created so we cannot authenticate.
        add_auth_header(req) unless Gem::Version.new(@om_version) >= Gem::Version.new('1.7')

        response = http.request(req)

        if Gem::Version.new(@om_version) == Gem::Version.new('1.7')
          wait_for_uaa_to_be_available

          ensure_env_admin_user_exists
        end

        response
      end

      def wait_for_uaa_to_be_available(sleep_interval = 10, max_retry_count = 24)
        retry_count = 0

        loop do
          request = Net::HTTP::Get.new(
            endpoints.login_ensure_availability
          )

          response = http.request(request)

          http_location = response['location']

          break if response.code == '302' && http_location == auth_redirect_location

          retry_count += 1

          if retry_count >= max_retry_count
            retry_wait_time = sleep_interval * retry_count

            raise("Timed out waiting for UAA to be available after #{retry_wait_time} seconds")
          end

          Kernel.sleep(sleep_interval)
        end
      end

      def ensure_env_admin_user_exists
        token = get_oauth_token('admin')

        create_env_admin_user(token) unless verify_if_env_admin_user_exists?(token)
      end

      def verify_if_env_admin_user_exists?(token)
        request = Net::HTTP::Get.new(
          endpoints.uaa_get_user_by_username_path(web_auth_user)
        )
        request['Authorization'] = token

        response = http.request(request)

        if response.code != '200'
          raise("Invalid response from UAA: #{response.code}")
        end

        reply = JSON.parse(response.body)

        if reply['resources'].nil?
          raise("Invalid response body from UAA: #{response.body}")
        end

        !reply['resources'].empty?
      end

      def create_env_admin_user(token)
        request = Net::HTTP::Post.new(
          endpoints.uaa_create_user
        )
        request['Content-Type'] = 'application/json'
        request['Authorization'] = token
        request.body = {
          userName: web_auth_user,
          password: web_auth_password,
          emails: [
            { value: web_auth_user }
          ]
        }.to_json

        response = http.request(request)

        if response.code != '201'
          raise("Unexpected response from UAA: code #{response.code}, body: #{response.body}")
        end
      end

      def import_secret_parameter_name
        Gem::Version.new(@om_version) >= Gem::Version.new('1.7') ? 'passphrase' : 'password'
      end

      def import_stemcell(stemcell_file)
        req = Net::HTTP::Post::Multipart.new(
          endpoints.import_stemcell_path,
          'stemcell[file]' =>
            UploadIO.new(
              stemcell_file,
              'application/octet-stream',
              File.basename(stemcell_file)
            )
        )

        add_auth_header(req)

        http.request(req)
      end

      def read_timeout
        environment.settings.dig('iaas_type') != 'vcloud' ? 600 : 3600
      end

      def auth_redirect_location
        host = environment.settings.dig('ops_manager', 'url')

        host.concat('/') unless host.end_with? '/'

        host+'auth/cloudfoundry'
      end

      private

      def add_auth_header(request)
        if Gem::Version.new(@om_version) >= Gem::Version.new('1.7')
          log.info('using oauth')
          token = get_oauth_token(web_auth_user)
          request['Authorization'] = token
        else
          log.info('using basic auth')
          request.basic_auth(web_auth_user, web_auth_password)
        end
      end

      def get_oauth_token(user)
        target_url = environment.settings.dig('ops_manager', 'url') + '/uaa'
        token_issuer = CF::UAA::TokenIssuer.new(target_url, 'opsman', nil, skip_ssl_validation: true)
        token_issuer.owner_password_grant(user, web_auth_password).auth_header
      end

      def web_auth_user
        environment.settings.dig('ops_manager', 'username')
      end

      def web_auth_password
        environment.settings.dig('ops_manager', 'password')
      end

      def http
        http = Net::HTTP.new(uri.host, uri.port)
        if uri.scheme == 'https'
          http.use_ssl = true
          http.verify_mode = OpenSSL::SSL::VERIFY_NONE
        end
        http.read_timeout = read_timeout
        http
      end

      def get_with_basic_auth(path)
        req = Net::HTTP::Get.new(path)
        add_auth_header(req)

        http.request(req)
      end

      class FakeResponse < Struct.new(:code, :body)
      end
    end
  end
end
# Copyright (c) 2014-2015 Pivotal Software, Inc.
# All rights reserved.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
# INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
# USE OR OTHER DEALINGS IN THE SOFTWARE.
