require 'opsmgr/settings/microbosh/installation_settings'
require 'opsmgr/settings/microbosh/product_list'
require 'opsmgr/settings/microbosh/job_list'
require 'opsmgr/settings/microbosh/property_list'

module Opsmgr
  module Api
    class Result
      attr_reader :message

      def success?
        true
      end
    end

    class InstallStatusResult < Result
      attr_reader :status

      def initialize(status_hash)
        @status = status_hash['status']
        status.freeze

        fail(ArgumentError, "Unrecognized status: #{status}") unless %w(failed success running).include?(status)
      end

      def failed?
        @status == 'failed'
      end

      def pending?
        @status == 'running'
      end

      def finished_successfully?
        @status == 'success'
      end

      def finished?
        failed? || finished_successfully?
      end
    end

    class InstallationDeletionStatusResult < Result
      NO_INSTALLATION = 'There is no installation.'.freeze
      DELETE_IN_PROGRESS = 'Delete in progress'.freeze

      attr_reader :status

      def initialize(status_hash)
        @status = status_hash['status']
      end

      def in_progress?
        status == DELETE_IN_PROGRESS
      end
    end

    class InstallLogResult < Result
      attr_reader :log

      def initialize(log_hash)
        @log = log_hash['logs'].dup.freeze
      end
    end

    class TriggerInstallResult < Result
      attr_reader :install_id

      def initialize(install_hash)
        @install_id = install_hash['install']['id']
      end
    end

    class ListProductsResult < Result
      Product = Struct.new(:name, :product_version)

      attr_reader :products

      def initialize(product_hash_array)
        @products = product_hash_array.map { |p| Product.new(p.fetch('name'), p.fetch('product_version')) }
      end
    end

    class InstalledProductsResult < Result
      def initialize(installed_products)
        @installed_products = installed_products
      end

      def guid_for(product_type)
        product_result = installed_products.find { |h| h.name == product_type }
        product_result ? product_result.fetch('guid') : nil
      end

      private

      attr_reader :installed_products
    end

    class InstallationSettingsResult < Result
      def initialize(install_hash)
        @install_hash = install_hash
      end

      def director_ip
        ips_for_job(product_name: 'p-bosh', job_name: 'director').first ||
          ips_for_job(product_name: 'microbosh', job_name: 'director').first
      end

      def ips_for_job(product_name:, job_name:)
        product = products.find { |p| p.name == product_name }
        return [] if product.nil?

        if install_hash['ip_assignments']
          install_hash['ip_assignments']['assignments'][product['guid']].each do |job_guid, assignment|
            return assignment.values.flatten if job_guid.starts_with?(job_name)
          end
        else
          product.fetch('ips').each do |job_guid, ips|
            return ips if job_guid.starts_with?(job_name)
          end
        end
        []
      end

      def director_password
        microbosh_jobs = microbosh_settings.fetch('jobs')
        microbosh_job_list = Opsmgr::Settings::Microbosh::JobList.new(microbosh_jobs)
        director_job = microbosh_job_list.find { |job| job.name == 'director' }
        properties = director_job.fetch('properties')
        property_list = Opsmgr::Settings::Microbosh::PropertyList.new(properties)
        credential_property = property_list.find { |property| property.name == 'director_credentials' }
        credential_property.fetch('value').fetch('password')
      end

      def uaa_admin_password
        jobs = cf_settings.fetch('jobs')
        job_list = Opsmgr::Settings::Microbosh::JobList.new(jobs)
        uaa_job = job_list.find { |job| job.name == 'uaa' }
        properties = uaa_job.fetch('properties')
        property_list = Opsmgr::Settings::Microbosh::PropertyList.new(properties)
        credential_property = property_list.find { |property| property.name == 'admin_credentials' }
        credential_property.fetch('value').fetch('password')
      end

      def system_domain
        properties = cc_job.fetch('properties')
        property_list = Opsmgr::Settings::Microbosh::PropertyList.new(properties)
        credential_property = property_list.find { |property| property.name == 'system_domain' }
        credential_property.fetch('value')
      end

      def apps_domain
        properties = cc_job.fetch('properties')
        property_list = Opsmgr::Settings::Microbosh::PropertyList.new(properties)
        credential_property = property_list.find { |property| property.name == 'apps_domain' }
        credential_property.fetch('value')
      end

      def product_version(product_guid)
        product = products.find { |p| p.guid == product_guid }
        return nil if product.nil?
        product.fetch('product_version')
      end

      def product_version_by_name(product_name)
        product = products.find { |p| p.name == product_name }
        return nil if product.nil?
        product.fetch('product_version')
      end

      def ops_manager_installation_settings
        Opsmgr::Settings::Microbosh::InstallationSettings.from_api_result(self)
      end

      def as_hash
        install_hash
      end

      private

      attr_reader :install_hash

      def microbosh_settings
        products.find { |product| product.name == 'p-bosh' || product.name == 'microbosh' } || fail('microbosh not found')
      end

      def cf_settings
        products.find { |product| product.name == 'cf' } || fail('cf not found')
      end

      def products
        if install_hash.key?('products')
          Opsmgr::Settings::Microbosh::ProductList.new(install_hash.fetch('products'))
        elsif install_hash.key?('components')
          Opsmgr::Settings::Microbosh::ProductList.new(install_hash.fetch('components'))
        else
          fail "Unable to find products in OpsManager response.  Check installation schema: #{install_hash.inspect}."
        end
      end

      def cc_job
        cf_settings.fetch('jobs')
        cf_jobs = cf_settings.fetch('jobs')
        cf_job_list = Opsmgr::Settings::Microbosh::JobList.new(cf_jobs)
        cf_job_list.find { |job| job.name == 'cloud_controller' }
      end
    end

    class ProductManifestResult < Result
      Product = Struct.new(:name, :version)
      Stemcell = Struct.new(:name, :version)

      def initialize(manifest_hash)
        @manifest_hash = manifest_hash
      end

      def releases
        manifest_hash.fetch('releases').map { |h| Product.new(h.fetch('name'), h.fetch('version')) }
      end

      def stemcell
        stemcell_hash = manifest_hash.fetch('resource_pools').first.fetch('stemcell')
        Stemcell.new(stemcell_hash.fetch('name'), stemcell_hash.fetch('version'))
      end

      private

      attr_reader :manifest_hash
    end

    class CreateFirstUserResult < Result
      attr_reader :user_id

      def initialize(user_hash)
        @user_id = user_hash['user']['id']
      end

      def user_already_existed?
        false
      end
    end

    class StagedManifestResult < Result
      attr_reader :manifest

      def initialize(manifest)
        @manifest = manifest
      end
    end

    class Error < Result
      def initialize(message, response)
        @message = "#{message}:\n#{response.body}"
        @response = response
      end

      def success?
        false
      end

      private

      attr_reader :response
    end

    class CreateFirstUserError < Error
      def user_already_existed?
        response.code == '401'
      end
    end
  end
end
# Copyright (c) 2014-2015 Pivotal Software, Inc.
# All rights reserved.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
# INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
# USE OR OTHER DEALINGS IN THE SOFTWARE.
