require 'aws-sdk-v1'
require 'mustache'
require 'active_support'
require 'active_support/core_ext'

module Opsmgr
  module Renderer
    class AWS < Mustache
      BASE_RENDERER_FIELDS = {
        vpc: 'PcfVpc',
        iam_aws_access_key_id: 'PcfIamUserAccessKey',
        iam_aws_secret_access_key: 'PcfIamUserSecretAccessKey',
        ssh_key_pair_name: 'PcfKeyPairName',
        public_subnet_id: 'PcfPublicSubnetId',
        private_subnet_id: 'PcfPrivateSubnetId',
        ops_manager_security_group_id: 'PcfOpsManagerSecurityGroupId',
        vms_security_group_id: 'PcfVmsSecurityGroupId',
        private_subnet_availability_zone: 'PcfPrivateSubnetAvailabilityZone',
        public_subnet_availability_zone: 'PcfPublicSubnetAvailabilityZone',
        s3_bucket_name: 'PcfOpsManagerS3Bucket',
        ert_s3_bucket_name: 'PcfElasticRuntimeS3Bucket',
        ert_s3_buildpacks_bucket_name: 'PcfElasticRuntimeS3BuildpacksBucket',
        ert_s3_droplets_bucket_name: 'PcfElasticRuntimeS3DropletsBucket',
        ert_s3_packages_bucket_name: 'PcfElasticRuntimeS3PackagesBucket',
        ert_s3_resources_bucket_name: 'PcfElasticRuntimeS3ResourcesBucket',
        rds_address: 'PcfRdsAddress',
        rds_port: 'PcfRdsPort',
        rds_username: 'PcfRdsUsername',
        rds_password: 'PcfRdsPassword',
        rds_ops_manager_dbname: 'PcfRdsDBName',
        ert_elb_dns_name: 'PcfElbDnsName',
        ert_ssh_elb_dns_name: 'PcfElbSshDnsName',
        ert_tcp_elb_dns_name: 'PcfElbTcpDnsName'
      }.freeze

      BASE_RENDERER_FIELDS.each do |method_sym, output_key|
        define_method(method_sym) do
          output(output_key)
        end
      end

      attr_reader :stack_name, :region
      def initialize(settings_file_contents)
        @settings_file_contents = settings_file_contents
        settings = YAML.load(@settings_file_contents)
        @vm_configs = settings.fetch('vm_shepherd').fetch('vm_configs')
        @stack_name = settings.fetch('vm_shepherd').fetch('env_config').fetch('stack_name')
        @region = settings.fetch('vm_shepherd').fetch('env_config').fetch('region')

        ::AWS.config(
          access_key_id: settings.fetch('vm_shepherd').fetch('env_config').fetch('aws_access_key'),
          secret_access_key: settings.fetch('vm_shepherd').fetch('env_config').fetch('aws_secret_key'),
          region: region,
        )
        @cloud_formation = ::AWS::CloudFormation.new
        @ec2 = ::AWS::EC2.new

        @custom_renderer_fields = settings.fetch('custom_renderer_fields', [])
        setup_custom_renderer_field_methods! unless @custom_renderer_fields.empty?
      end

      def setup_custom_renderer_field_methods!
        @custom_renderer_fields.each do |property|
          method_body = proc do
            # convert my_custom_key to MyCustomKey to match AWS format
            output(property.camelize)
          end

          self.class.send(:define_method, property.to_sym, &method_body)
        end
      end

      def rendered_settings
        render(@settings_file_contents)
      end

      def vms_security_group_name
        security_group = @ec2.security_groups.find { |g| g.id == vms_security_group_id }
        security_group ? security_group.name : ''
      end

      def s3_endpoint
        {
          'us-east-1' => 'https://s3.amazonaws.com',
          'us-west-1' => 'https://s3-us-west-1.amazonaws.com',
        }[region]
      end

      def release_candidate_public_ip
        ops_manager_elastic_ip ? ops_manager_elastic_ip.public_ip : nil
      end

      def previous_version_public_ip
        previous_version_elastic_ip ? previous_version_elastic_ip.public_ip : nil
      end

      private

      def output(key)
        output = outputs.detect { |o| o.key == key }
        return output.value unless output.nil?
        ''
      rescue ::AWS::CloudFormation::Errors::ValidationError
        ''
      end

      def outputs
        @outputs ||= @cloud_formation.stacks[@stack_name].outputs
      rescue ::AWS::CloudFormation::Errors::ValidationError
        @outputs = {}
      end

      def ops_manager_elastic_ip
        @ops_manager_elastic_ip ||= ops_manager_instance ? ops_manager_instance.elastic_ip : nil
      end

      def ops_manager_instance
        @ops_manager_instance ||= find_vm_instance('RELEASE-CANDIATE')
      end

      def previous_version_elastic_ip
        @previous_version_elastic_ip ||= previous_version_instance ? previous_version_instance.elastic_ip : nil
      end

      def previous_version_instance
        @previous_version_instance ||= find_vm_instance('PREVIOUS-VERSION')
      end

      def find_vm_instance(vm_name_token)
        @ec2.instances.find do |instance|
          instance.tags.to_h['Name'] == find_vm_name(vm_name_token) && instance.status == :running
        end
      end

      def find_vm_name(vm_name_token)
        @vm_configs.find { |config| config.fetch('vm_name').match(vm_name_token) }.fetch('vm_name')
      end
    end
  end
end
# Copyright (c) 2014-2015 Pivotal Software, Inc.
# All rights reserved.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
# INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
# USE OR OTHER DEALINGS IN THE SOFTWARE.
